/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    See file, 'COPYING', for details.
*/


/*
 * some really really really really simple code
 * to write out quake1 map info.
 */


// uses some Id Software stuff...

#include "cmdlib.h"
#include "mathlib.h"
#include "bspfile.h"

// very dumb write out procedure
void writeOutBSP(char* filename){

    int i;
    char a,b,c;
    short myvertexnum;
    short mynodenum;
    short myfacenum;
    short myedgenum;
    short mysedgesnum;
    short myplanenum;

    FILE* out = fopen(filename,"wb");
    if(!out)
        Error("cannot open %s for writing.",filename);

    // write planes

    myplanenum = BigShort((short)numplanes);
    fwrite(&myplanenum,sizeof(myplanenum),1,out);
    for(i=0;i<numplanes;i++){
        dplane_t* plane = &dplanes[i];
        short dist;
        if(plane->normal[0] * 10000 == 0)
            plane->normal[0] = 0.0;
        if(plane->normal[1] * 10000 == 0)
            plane->normal[1] = 0.0;
        if(plane->normal[2] * 10000 == 0)
            plane->normal[2] = 0.0;

        a = (char)(plane->normal[0] * 127);
        b = (char)(plane->normal[1] * 127);
        c = (char)(plane->normal[2] * 127);

        // 5 bytes per plane

        fwrite(&a,sizeof(a),1,out);
        fwrite(&b,sizeof(b),1,out);
        fwrite(&c,sizeof(c),1,out);

        dist = BigShort((short)plane->dist);
        fwrite(&dist,sizeof(dist),1,out);
    }

    // write vertices

    myvertexnum = BigShort((short)numvertexes);
    fwrite(&myvertexnum,sizeof(myvertexnum),1,out);
    for(i=0;i<numvertexes;i++){
        dvertex_t* vertex = &dvertexes[i];
        short x = BigShort((short)((int)vertex->point[0]));
        short y = BigShort((short)((int)vertex->point[1]));
        short z = BigShort((short)((int)vertex->point[2]));

        // 6 bytes per vertex

        fwrite(&x,sizeof(x),1,out);
        fwrite(&y,sizeof(y),1,out);
        fwrite(&z,sizeof(z),1,out);
    }

    // write bsp nodes

    mynodenum = BigShort((short)numnodes);
    fwrite(&mynodenum,sizeof(mynodenum),1,out);
    for(i=0;i<numnodes;i++){
        dnode_t* node = &dnodes[i];     // negative children are -(leafs+1), not nodes

        // 9 bytes per node

        short a = BigShort(node->planenum);
        short b = BigShort(node->children[0]);
        short c = BigShort(node->children[1]);
        short d = BigShort(node->firstface);
        char  e = (char)node->numfaces;

        fwrite(&a,sizeof(a),1,out);
        fwrite(&b,sizeof(b),1,out);
        fwrite(&c,sizeof(c),1,out);
        fwrite(&d,sizeof(d),1,out);
        fwrite(&e,sizeof(e),1,out);
    }

    // write faces

    myfacenum = BigShort((short)numfaces);
    fwrite(&myfacenum,sizeof(myfacenum),1,out);
    for(i=0;i<numfaces;i++){
        dface_t* dface = &dfaces[i];
        short a = BigShort(dface->firstedge);
        char b = dface->numedges;
        if(dface->side)    // side is encoded as the most significant bit of numedges
            b |= 128;

        fwrite(&a,sizeof(a),1,out);     // 3 bytes per face
        fwrite(&b,sizeof(b),1,out);
    }

    // write edges

    myedgenum = BigShort((short)numedges);
    fwrite(&myedgenum,sizeof(myedgenum),1,out);
    for(i=0;i<numedges;i++){
        dedge_t* dedge = &dedges[i];
        short a = BigShort((short)dedge->v[0]);
        short b = BigShort((short)dedge->v[1]);

        // 4 bytes per edge

        fwrite(&a,sizeof(a),1,out);
        fwrite(&b,sizeof(b),1,out);
    }

    // write surface edges

    mysedgesnum = BigShort((short)numsurfedges);
    fwrite(&mysedgesnum,sizeof(mysedgesnum),1,out);

    for(i=0;i<numsurfedges;i++){
        short a = BigShort((short)dsurfedges[i]);
        fwrite(&a,sizeof(a),1,out);
    }

    fclose(out);
}


int main (int argc, char **argv)
{
    int         i;
    char        source[1024];

    if (argc < 3)
        Error ("usage: bsp2pbsp inputbspfile outputfile");

    strcpy(source,argv[1]);
    DefaultExtension(source, ".bsp");
    LoadBSPFile(source);

    writeOutBSP(argv[2]);

    return 0;
}

